@@ -16,17 +16,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <gtk/gtk.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "gettext.h"
 #include "wavbreaker.h"
 
-#define COPYRIGHT "Copyright (C) 2002-2007 Timothy Robinson\nCopyright (C) 2006-2008 Thomas Perl"
+#define COPYRIGHT "Copyright (C) 2002-2007 Timothy Robinson\nCopyright (C) 2006-2008, 2012, 2015-2016, 2018-2019 Thomas Perl"
 #define APPDESCRIPTION _("Split a wave file into multiple chunks")
 #define URL "http://wavbreaker.sourceforge.net/"
 
@@ -36,11 +33,8 @@
 
 void about_show(GtkWidget *main_window)
 {
-        GdkPixbuf* icon;
         const char* authors[] = { AUTHOR_A, AUTHOR_B, 0 };
 
-        icon = gdk_pixbuf_new_from_file( WAVBREAKER_ICON, NULL);
-
         gtk_show_about_dialog (GTK_WINDOW(main_window),
                           "name", PACKAGE,
                           "version", VERSION,
@@ -49,7 +43,7 @@ void about_show(GtkWidget *main_window)
                           "website", URL,
                           "website-label", URL,
                           "authors", authors,
-                          "logo", icon,
+                          "logo-icon-name", "net.sourceforge.wavbreaker",
                           NULL);
 }
 
