@@ -21,27 +21,28 @@
 #define APPCONFIG_H
 
 #include "sample.h"
+#include "wavbreaker.h"
 
 typedef struct AudioFunctionPointers_ AudioFunctionPointers;
 
 struct AudioFunctionPointers_ {
     void (*audio_close_device)();
-    int (*audio_open_device)(const char *, SampleInfo *);
+    int (*audio_open_device)(SampleInfo *);
     int (*audio_write)(unsigned char *, int);
-    char *(*get_outputdev)();
 };
 
 void appconfig_show(GtkWidget *);
-int appconfig_write_file();
+void appconfig_write_file();
 void appconfig_init();
 
-int get_use_outputdir();
-char *get_outputdir();
 AudioFunctionPointers *get_audio_function_pointers();
-int get_use_etree_filename_suffix();
-int get_prepend_file_number();
-char *get_etree_filename_suffix();
-char *get_etree_cd_length();
+
+int appconfig_get_use_outputdir();
+char *appconfig_get_outputdir();
+int appconfig_get_use_etree_filename_suffix();
+int appconfig_get_prepend_file_number();
+char *appconfig_get_etree_filename_suffix();
+char *appconfig_get_etree_cd_length();
 
 int appconfig_get_main_window_xpos();
 void appconfig_set_main_window_xpos(int x);
@@ -57,16 +58,8 @@ int appconfig_get_vpane2_position();
 void appconfig_set_vpane2_position(int x);
 int appconfig_get_silence_percentage();
 void appconfig_set_silence_percentage(int x);
-int appconfig_get_ask_really_quit();
-void appconfig_set_ask_really_quit(int x);
-int appconfig_get_show_toolbar();
-void appconfig_set_show_toolbar(int x);
 int appconfig_get_show_moodbar();
 void appconfig_set_show_moodbar(int x);
 
-char *audio_options_get_output_device();
-void set_audio_oss_options_output_device(const char *);
-void set_audio_alsa_options_output_device(const char *);
-
 #endif /* APPCONFIG_H */
 
