@@ -72,17 +72,6 @@ int get_merge_files_count()
     return i;
 }
 
-static void guimerge_hide( GtkWidget *main_window)
-{
-    gtk_list_store_clear( store);
-    gtk_widget_destroy( main_window);
-}
-
-static void cancel_button_clicked( GtkWidget *widget, gpointer user_data)
-{
-    guimerge_hide( GTK_WIDGET(user_data));
-}
-
 static void ok_button_clicked(GtkWidget *widget, gpointer user_data)
 {
     GtkWidget *dialog;
@@ -100,7 +89,7 @@ static void ok_button_clicked(GtkWidget *widget, gpointer user_data)
         memset (&value, 0, sizeof (GValue));
         gtk_tree_model_get_value( GTK_TREE_MODEL(store), &iter, 0, &value);
         tmp = (char*)g_value_peek_pointer( &value);
-        filenames = g_list_append(filenames, strdup(tmp));
+        filenames = g_list_append(filenames, g_strdup(tmp));
     } while( gtk_tree_model_iter_next( GTK_TREE_MODEL(store), &iter) == TRUE);
 
     filter_all = gtk_file_filter_new();
@@ -114,8 +103,8 @@ static void ok_button_clicked(GtkWidget *widget, gpointer user_data)
     dialog = gtk_file_chooser_dialog_new( _("Select filename for merged wave file"),
                                           GTK_WINDOW(window),
                                           GTK_FILE_CHOOSER_ACTION_SAVE,
-                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                          GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                          _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                          _("_Save"), GTK_RESPONSE_ACCEPT,
                                           NULL);
 
     gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(dialog), filter_all);
@@ -125,7 +114,7 @@ static void ok_button_clicked(GtkWidget *widget, gpointer user_data)
     gtk_file_chooser_set_do_overwrite_confirmation( GTK_FILE_CHOOSER(dialog), TRUE);
 
     checkbutton = (GtkWidget*)gtk_check_button_new_with_label( _("Open file in wavbreaker after merge"));
-    gtk_box_pack_end( GTK_BOX(GTK_DIALOG(dialog)->vbox), checkbutton, FALSE, FALSE, 0);
+    gtk_box_pack_end(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), checkbutton, FALSE, FALSE, 0);
     gtk_widget_show( GTK_WIDGET(checkbutton));
 
     if( strlen( folder) > 0) {
@@ -138,7 +127,7 @@ static void ok_button_clicked(GtkWidget *widget, gpointer user_data)
         tmp = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER(dialog));
         write_info.pct_done = 0.0;
         sample_merge_files( tmp, filenames, &write_info);
-        guimerge_hide( GTK_WIDGET(user_data));
+        gtk_widget_destroy(GTK_WIDGET(user_data));
         g_idle_add( file_merge_progress_idle_func, NULL);
     }
 
@@ -164,7 +153,10 @@ static void add_filename( char* filename)
             common_sample_info.samplesPerSec != sampleinfo.samplesPerSec ||
             common_sample_info.avgBytesPerSec != sampleinfo.avgBytesPerSec ||
             common_sample_info.blockAlign != sampleinfo.blockAlign ||
-            common_sample_info.bitsPerSample != sampleinfo.bitsPerSample) {
+            common_sample_info.bitsPerSample != sampleinfo.bitsPerSample ||
+            sampleinfo.channels == 0 ||
+            sampleinfo.samplesPerSec == 0 ||
+            sampleinfo.bitsPerSample < 8) {
             popupmessage_show( window, _("Wrong file format - skipping file"), filename);
             return;
         }
@@ -201,8 +193,10 @@ static void add_button_clicked( GtkWidget *widget, gpointer user_data)
     gtk_file_filter_add_pattern( filter_supported, "*.wav");
 
     dialog = gtk_file_chooser_dialog_new(_("Add wave file to merge"), GTK_WINDOW(window),
-        GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+        GTK_FILE_CHOOSER_ACTION_OPEN,
+        _("_Cancel"), GTK_RESPONSE_CANCEL,
+        _("_Open"), GTK_RESPONSE_ACCEPT,
+        NULL);
 
     gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(dialog), filter_all);
     gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(dialog), filter_supported);
@@ -215,7 +209,10 @@ static void add_button_clicked( GtkWidget *widget, gpointer user_data)
     }
 
     if (gtk_dialog_run( GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
-        strcpy( folder, gtk_file_chooser_get_current_folder( GTK_FILE_CHOOSER(dialog)));
+        const char *current_folder = gtk_file_chooser_get_current_folder( GTK_FILE_CHOOSER(dialog));
+        if (current_folder) {
+            strcpy(folder, current_folder);
+        }
 
         GSList* filenames;
         filenames = gtk_file_chooser_get_filenames( GTK_FILE_CHOOSER(dialog));
@@ -260,50 +257,40 @@ void guimerge_show( GtkWidget *main_window)
 {
     GtkWidget *vbox;
     GtkWidget *hbox;
-    GtkWidget *hbbox;
     GtkWidget *vbbox;
-    GtkWidget *message_label;
-    GtkWidget *cancel_button;
     GtkWidget *button;
-    GtkWidget *button_hbox;
 
     GtkTreeSelection *selection;
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
     GtkWidget *sw;
 
-    char tmpstr[4096];
-
     window = gtk_window_new( GTK_WINDOW_TOPLEVEL);
 
+    GtkWidget *header_bar = gtk_header_bar_new();
+    gtk_header_bar_set_show_close_button(GTK_HEADER_BAR(header_bar), TRUE);
+    gtk_header_bar_set_title(GTK_HEADER_BAR(header_bar), _("Merge wave files"));
+    gtk_window_set_titlebar(GTK_WINDOW(window), header_bar);
+
     gtk_widget_realize( window);
     gtk_window_set_modal( GTK_WINDOW(window), TRUE);
     gtk_window_set_transient_for( GTK_WINDOW(window), GTK_WINDOW(main_window));
     gtk_window_set_type_hint( GTK_WINDOW(window), GDK_WINDOW_TYPE_HINT_DIALOG);
     gtk_window_set_position( GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
-    gtk_window_set_title( GTK_WINDOW(window), _("Merge wave files"));
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add(GTK_CONTAINER(window), vbox);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
     gtk_box_set_spacing( GTK_BOX(vbox), 6);
 
-    sprintf( tmpstr, "<big><b>%s</b></big>", gtk_window_get_title( GTK_WINDOW(window)));
-
-    message_label = gtk_label_new( NULL);
-    gtk_label_set_markup( GTK_LABEL(message_label), tmpstr);
-    gtk_misc_set_alignment( GTK_MISC(message_label), 0, 0.5);
-    gtk_box_pack_start( GTK_BOX(vbox), message_label, FALSE, TRUE, 0);
-
-    message_label = gtk_label_new( _("Add wave files to this list and click on \"Merge\"."));
-    gtk_misc_set_alignment( GTK_MISC(message_label), 0, 0.5);
-    gtk_box_pack_start( GTK_BOX(vbox), message_label, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_set_spacing( GTK_BOX(hbox), 6);
     gtk_box_pack_start( GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
-    store = gtk_list_store_new( NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING);
+    if (!store) {
+        store = gtk_list_store_new( NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING);
+    }
+    gtk_list_store_clear(store);
 
     /* create the scrolled window for the list */
     sw = gtk_scrolled_window_new( NULL, NULL);
@@ -320,6 +307,7 @@ void guimerge_show( GtkWidget *main_window)
     column = gtk_tree_view_column_new();
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_column_set_title( column, _("File Name"));
+    gtk_tree_view_column_set_expand(column, TRUE);
     gtk_tree_view_column_pack_start( column, renderer, TRUE);
     gtk_tree_view_column_add_attribute(column, renderer, "text", COLUMN_BASENAME);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
@@ -338,42 +326,26 @@ void guimerge_show( GtkWidget *main_window)
 
     gtk_box_pack_start( GTK_BOX(hbox), sw, TRUE, TRUE, 0);
 
-    vbbox = gtk_vbox_new( FALSE, 0);
+    vbbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_box_pack_start( GTK_BOX(hbox), vbbox, FALSE, FALSE, 0);
     gtk_box_set_spacing(GTK_BOX(vbbox), 5);
 
-    button = gtk_button_new_from_stock( GTK_STOCK_ADD);
+    button = gtk_button_new_with_mnemonic(_("_Add"));
     gtk_box_pack_start( GTK_BOX(vbbox), button, FALSE, FALSE, 0);
-    g_signal_connect( G_OBJECT(button), "clicked", (GtkSignalFunc)add_button_clicked, window);
+    g_signal_connect( G_OBJECT(button), "clicked", (GCallback)add_button_clicked, window);
 
-    remove_button = gtk_button_new_from_stock( GTK_STOCK_REMOVE);
+    remove_button = gtk_button_new_with_mnemonic(_("_Remove"));
     gtk_box_pack_start( GTK_BOX(vbbox), remove_button, FALSE, FALSE, 0);
-    g_signal_connect( G_OBJECT(remove_button), "clicked", (GtkSignalFunc)remove_button_clicked, window);
+    g_signal_connect( G_OBJECT(remove_button), "clicked", (GCallback)remove_button_clicked, window);
     gtk_widget_set_sensitive( GTK_WIDGET(remove_button), FALSE);
 
-    gtk_box_pack_start( GTK_BOX(vbbox), gtk_hseparator_new(), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbbox), gtk_label_new(""), TRUE, TRUE, 0);
 
-    ok_button = gtk_button_new();
-    g_signal_connect(G_OBJECT(ok_button), "clicked", (GtkSignalFunc)ok_button_clicked, window);
+    ok_button = gtk_button_new_with_label(_("Merge"));
+    g_signal_connect(G_OBJECT(ok_button), "clicked", (GCallback)ok_button_clicked, window);
     gtk_box_pack_start( GTK_BOX(vbbox), ok_button, FALSE, FALSE, 0);
     gtk_widget_set_sensitive( GTK_WIDGET(ok_button), FALSE);
 
-    button_hbox = gtk_hbox_new( FALSE, 0);
-    gtk_container_add( GTK_CONTAINER(ok_button), button_hbox);
-    gtk_box_pack_start( GTK_BOX(button_hbox), gtk_image_new_from_stock( GTK_STOCK_SAVE, GTK_ICON_SIZE_BUTTON), TRUE, TRUE, 0);
-    gtk_box_pack_start( GTK_BOX(button_hbox), gtk_label_new( _("Merge")), TRUE, TRUE, 0);
-
-    gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
-
-    hbbox = gtk_hbutton_box_new();
-    gtk_box_pack_start( GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbbox), GTK_BUTTONBOX_END);
-    gtk_box_set_spacing(GTK_BOX(hbbox), 10);
-
-    cancel_button = gtk_button_new_from_stock( GTK_STOCK_CLOSE);
-    gtk_box_pack_end(GTK_BOX(hbbox), cancel_button, FALSE, FALSE, 5);
-    g_signal_connect(G_OBJECT(cancel_button), "clicked", (GtkSignalFunc)cancel_button_clicked, window);
-
     gtk_window_resize( GTK_WINDOW(window), 500, 300);
     gtk_widget_show_all(window);
 }
@@ -384,14 +356,10 @@ gboolean file_merge_progress_idle_func(gpointer data) {
     static GtkWidget *vbox;
     static GtkWidget *label;
     static GtkWidget *status_label;
-    static char tmp_str[6144];
-    static char str[6144];
-    char *str_ptr;
     static int cur_file_displayed = 0;
     static double fraction;
 
     if (window == NULL) {
-        gdk_threads_enter();
         window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
         gtk_widget_realize(window);
         gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
@@ -402,91 +370,73 @@ gboolean file_merge_progress_idle_func(gpointer data) {
                 GDK_WINDOW_TYPE_HINT_DIALOG);
         gtk_window_set_position(GTK_WINDOW(window),
                 GTK_WIN_POS_CENTER_ON_PARENT);
-        gdk_window_set_functions(window->window, GDK_FUNC_MOVE);
+        gdk_window_set_functions(gtk_widget_get_window(window), GDK_FUNC_MOVE);
 
-
-        vbox = gtk_vbox_new(FALSE, 0);
+        vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
         gtk_container_add(GTK_CONTAINER(window), vbox);
         gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
 
         gtk_window_set_title( GTK_WINDOW(window), _("Merging wave files"));
 
-        tmp_str[0] = '\0';
-        strcat( tmp_str, "<span size=\"larger\" weight=\"bold\">");
-        strcat( tmp_str, gtk_window_get_title( GTK_WINDOW(window)));
-        strcat( tmp_str, "</span>");
-
-        label = gtk_label_new( NULL);
-        gtk_label_set_markup( GTK_LABEL(label), tmp_str);
-        gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5);
+        gchar *markup = g_markup_printf_escaped("<span size=\"larger\" weight=\"bold\">%s</span>",
+                gtk_window_get_title(GTK_WINDOW(window)));
+        label = gtk_label_new(NULL);
+        gtk_label_set_markup(GTK_LABEL(label), markup);
+        g_free(markup);
+        g_object_set(G_OBJECT(label), "xalign", 0.0f, "yalign", 0.5f, NULL);
         gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 5);
 
         label = gtk_label_new( _("The selected files are now being merged. This can take some time."));
         gtk_label_set_line_wrap( GTK_LABEL(label), TRUE);
-        gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5);
+        g_object_set(G_OBJECT(label), "xalign", 0.0f, "yalign", 0.5f, NULL);
         gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 5);
 
         pbar = gtk_progress_bar_new();
         gtk_box_pack_start(GTK_BOX(vbox), pbar, FALSE, TRUE, 5);
 
         status_label = gtk_label_new( NULL);
-        gtk_misc_set_alignment( GTK_MISC(status_label), 0.0, 0.5);
+        g_object_set(G_OBJECT(label), "xalign", 0.0f, "yalign", 0.5f, NULL);
         gtk_label_set_ellipsize( GTK_LABEL(status_label), PANGO_ELLIPSIZE_MIDDLE);
         gtk_box_pack_start(GTK_BOX(vbox), status_label, FALSE, TRUE, 5);
 
         gtk_widget_show_all(GTK_WIDGET(window));
         cur_file_displayed = -1;
-
-        gdk_threads_leave();
     }
 
     if (write_info.sync) {
-        gdk_threads_enter();
-
         write_info.sync = 0;
         gtk_widget_destroy(window);
         window = NULL;
 
-        sprintf( tmp_str, _("The files have been merged as %s."), basename(write_info.merge_filename));
-        popupmessage_show( NULL, _("Operation successful"), tmp_str);
+        popupmessage_show(NULL, _("Operation successful"), _("The files have been merged."));
 
-        gdk_threads_leave();
         return FALSE;
     }
 
     if (cur_file_displayed != write_info.cur_file) {
-        gdk_threads_enter();
-
-        str_ptr = basename( write_info.cur_filename);
-
-        if( str_ptr == NULL) {
-            str_ptr = write_info.cur_filename;
-        }
-
-        sprintf( str, _("Adding %s"), str_ptr);
-        sprintf( tmp_str, "<i>%s</i>", str);
-        gtk_label_set_markup(GTK_LABEL(status_label), tmp_str);
+        gchar *bn = g_path_get_basename(write_info.cur_filename);
+        gchar *tmp = g_strdup_printf(_("Adding %s"), bn);
+        g_free(bn);
+        gchar *msg = g_markup_printf_escaped("<i>%s</i>", tmp);
+        g_free(tmp);
+        gtk_label_set_markup(GTK_LABEL(status_label), msg);
+        g_free(msg);
 
         cur_file_displayed = write_info.cur_file;
-
-        gdk_threads_leave();
     }
 
-    gdk_threads_enter();
-
     fraction = 1.00*(write_info.cur_file-1+write_info.pct_done)/write_info.num_files;
     gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), fraction);
 
-    if( write_info.num_files > 1) {
-        sprintf( tmp_str, _("%d of %d files merged"), write_info.cur_file-1, write_info.num_files);
+    gchar *msg;
+    if (write_info.num_files > 1) {
+        // TODO: i18n plural forms
+        msg = g_strdup_printf(_("%d of %d files merged"), write_info.cur_file-1, write_info.num_files);
     } else {
-        sprintf( tmp_str, _("%d of 1 file merged"), write_info.cur_file-1);
+        msg = g_strdup_printf(_("%d of 1 file merged"), write_info.cur_file-1);
     }
-    gtk_progress_bar_set_text( GTK_PROGRESS_BAR(pbar), tmp_str);
-
-    gdk_threads_leave();
-
-    usleep( 100000);
+    gtk_progress_bar_set_text( GTK_PROGRESS_BAR(pbar), msg);
+    g_free(msg);
 
     return TRUE;
 }
