@@ -20,7 +20,6 @@
 #define WAV_H
 
 #include "sample.h"
-#include "wavbreaker.h"
 
 #define RiffID "RIFF"
 #define WaveID "WAVE"
@@ -50,11 +49,16 @@ typedef struct {
 //	unsigned short  extraNonPcm;
 } FormatChunk;
 
-char *wav_get_error_message();
+const char *wav_get_error_message();
 
 int wav_read_header(char *, SampleInfo *, int);
 int wav_read_sample(FILE *, unsigned char *, int, unsigned long);
 
+int
+wav_write_file_header(FILE *fp,
+                      SampleInfo *sample_info,
+                      unsigned long num_bytes);
+
 int
 wav_write_file(FILE *fp,
                char *filename,
