@@ -20,66 +20,29 @@
 #ifndef WAVBREAKER_H
 #define WAVBREAKER_H
 
-#define WAVBREAKER_ICON ICONDIR"/wavbreaker.png"
-
 #include <gtk/gtk.h>
 
-typedef struct Points_ Points;
-struct Points_ {
-        int min, max;
-};
-
-typedef struct GraphData_ GraphData;
-struct GraphData_{
-	unsigned long numSamples;
-	unsigned long maxSampleValue;
-        unsigned long maxSampleAmp;
-        unsigned long minSampleAmp;
-	Points *data;
-};
-
-typedef struct MoodbarData_ MoodbarData;
-struct MoodbarData_ {
-    unsigned long numFrames;
-    GdkColor *frames;
-};
-
 typedef struct TrackBreak_ TrackBreak;
 struct TrackBreak_ {
 	gboolean  write;
-	guint     offset;
+	gulong    offset;
 	gchar     *filename;
 	gchar     time[128];
 	gchar     duration[128];
 	gboolean  editable;
 };
 
-typedef struct WriteInfo_ WriteInfo;
-struct WriteInfo_ {
-	guint num_files;
-	guint cur_file;
-	char *cur_filename;
-	char *merge_filename;
-	double pct_done;
-	guint sync;
-	gint check_file_exists;
-	gint skip_file; /* -1 = waiting for check
-                     * 0 = don't overwrite file
-                     * 1 = file is ok to overwrite
-                     * 2 = overwrite all files
-                     */
-    gint sync_check_file_overwrite_to_write_progress;
-};
-
 void wavbreaker_autosplit(long x);
 void track_break_rename();
 void wavbreaker_write_files(char *dirname);
 GtkWidget *wavbreaker_get_main_window();
-gboolean open_file_arg( gpointer data);
-void track_break_add_offset( char* filename, guint offset);
+void track_break_add_offset(char *filename, gulong offset);
 void track_break_clear_list();
+void wavbreaker_update_moodbar_state();
 void wavbreaker_quit();
 
+guint msf_time_to_offset( gchar *str );
+
 extern char *sample_filename;
 
 enum {
@@ -88,12 +51,8 @@ enum {
     CHECK_INVERT
 };
 
-#define MB_OVL_MOODBAR 2
-#define MB_OVL_WAVEFORM 7
-#define MOODBAR_BLEND(waveform,moodbar) (((MB_OVL_WAVEFORM*waveform+MB_OVL_MOODBAR*moodbar))/(MB_OVL_MOODBAR+MB_OVL_WAVEFORM))
-
 /**
- * When the play marker reaches (x-1)/x (where x is the 
+ * When the play marker reaches (x-1)/x (where x is the
  * value of PLAY_MARKER_SCROLL), scroll the waveform so
  * that the play marker continues at position 1/x.
  *
